### 生产者-消费者问题 {#一、生产者-消费者问题}

---

**问题描述：**

一组生产者进程和一组消费者进程共享一个初始为空、大小为 n 的缓冲区，只有缓冲区没满时，生产者才能把消息放入到缓冲区，否则必须等待；只有缓冲区不空时，消费者才能从中取出消息，否则必须等待。由于缓冲区是临界资源，它只允许一个生产者放入消息，或者一个消费者从中取出消息。

**分析：**

1. 关系分析：生产者和消费者对缓冲区互斥访问是互斥关系，同时生产者和消费者又是一个相互协作的关系，只有生产者生产之后，消费者才能消费，它们也是同步关系。

2. 整理思路：这里比较简单，只有生产者和消费者两个进程，且这两个进程存在着互斥关系和同步关系。那么需要解决的是互斥和同步的PV操作的位置。

3. 信号量设置：信号量`mutex`作为互斥信号量，用于控制互斥访问缓冲池，初值为1；信号量`full`用于记录当前缓冲池中“满”缓冲区数，初值为 0；信号量`empty`用于记录当前缓冲池中“空”缓冲区数，初值为n。

**Java代码示例**







### 读者-写者问题 {#二、读者-写者问题}

---

**问题描述：**

有读者和写者两组并发线程，共享一个文件，当两个或以上的读线程同时访问共享数据时不会产生副作用，但若某个写线程和其他线程（读线程或写线程）同时访问共享数据时则可能导致数据不一致的错误。因此要求：

* 允许多个读者可以同时对文件执行读操作；
* 只允许一个写者往文件中写信息；
* 任一写者在完成写操作之前不允许其他读者或写者工作；
* 写者执行写操作前，应让已有的读者和写者全部退出。

**分析：**

1. 关系分析：由题目分析可知，读者和写者是互斥的，写者和写者也是互斥的，而读者和读者不存在互斥问题。

2. 整理思路：写者是比较简单的，它与任何线程互斥，用互斥信号量的 PV 操作即可解决。读者的问题比较复杂，它必须实现与写者的互斥，多个读者还可以同时读。所以，在这里用到了一个计数器，用它来判断当前是否有读者读文件。当有读者的时候写者是无法写文件的，此时读者会一直占用文件，当没有读者的时候写者才可以写文件。同时，不同的读者对计数器的访问也应该是互斥的。

3. 信号量设置：首先设置一个计数器`count`，用来记录当前的读者数量，初值为0；设置互斥信号量`mutex`，用于保护更新 count 变量时的互斥；设置互斥信号量`rw`用于保证读者和写者的互斥访问。

**Java代码示例**

### 哲学家进餐问题 {#三、哲学家进餐问题}

---

**问题描述：**

一张圆桌上坐着 5 名哲学家，桌子上每两个哲学家之间摆了一根筷子，桌子的中间是一碗米饭，如下图所示：

![](/assets/3370246-f8d801f01cbbf0b3.jpg)

哲学家们倾注毕生精力用于思考和进餐，哲学家在思考时，并不影响他人。只有当哲学家饥饿的时候，才试图拿起左、右两根筷子（一根一根拿起）。如果筷子已在他人手上，则需等待。饥饿的哲学家只有同时拿到了两根筷子才可以开始进餐，当进餐完毕后，放下筷子继续思考。

**分析：**

1. 关系分析：5名哲学家与左右邻居对其中间筷子的访问是互斥关系。

2. 整理思路：显然这里有 5 个线程，那么要如何让一个哲学家拿到左右两个筷子而不造成死锁或饥饿现象？解决方法有两个，一个是让他们同时拿两个筷子；二是对每个哲学家的动作制定规则，避免饥饿或死锁现象的发生。

3. 信号量设置：定义互斥信号量数组`chopstick[5] = {1,1,1,1,1}`用于对 5 根筷子的互斥访问。

## 内容来源

[【Linux多线程】三个经典同步问题](https://songlee24.github.io/2015/04/30/linux-three-syn-problems/)

